@@ -24,6 +24,7 @@ from typing import (
 from debputy import DEBPUTY_ROOT_DIR, DEBPUTY_PLUGIN_ROOT_DIR
 from debputy.analysis import REFERENCE_DATA_TABLE
 from debputy.analysis.debian_dir import scan_debian_dir
+from debputy.build_support import perform_clean, perform_builds
 from debputy.commands.debputy_cmd.context import (
     CommandContext,
     add_arg,
@@ -40,10 +41,15 @@ from debputy.exceptions import (
     UnhandledOrUnexpectedErrorFromPluginError,
     SymlinkLoopError,
 )
+from debputy.highlevel_manifest import HighLevelManifest
 from debputy.package_build.assemble_deb import (
     assemble_debs,
 )
-from debputy.plugin.api.spec import INTEGRATION_MODE_DH_DEBPUTY_RRR
+from debputy.plugin.api.spec import (
+    INTEGRATION_MODE_DH_DEBPUTY_RRR,
+    DebputyIntegrationMode,
+    INTEGRATION_MODE_FULL,
+)
 
 try:
     from argcomplete import autocomplete
@@ -92,8 +98,9 @@ from debputy.util import (
     escape_shell,
     program_name,
     integrated_with_debhelper,
-    change_log_level,
+    PRINT_BUILD_SYSTEM_COMMAND,
     PRINT_COMMAND,
+    change_log_level,
 )
 
 
@@ -255,6 +262,18 @@ def _add_packages_args(parser: argparse.ArgumentParser) -> None:
     )
 
 
+def _build_subcommand_log_level(context: CommandContext) -> int:
+    parsed_args = context.parsed_args
+    log_level: Optional[int] = None
+    if os.environ.get("DH_VERBOSE", "") != "":
+        log_level = PRINT_COMMAND
+    if parsed_args.debug_mode:
+        log_level = logging.INFO
+    if log_level is not None:
+        change_log_level(log_level)
+    return PRINT_BUILD_SYSTEM_COMMAND
+
+
 internal_commands = ROOT_COMMAND.add_dispatching_subcommand(
     "internal-command",
     dest="internal_command",
@@ -629,11 +648,72 @@ def _run_tests_for_plugin(context: CommandContext) -> None:
     _run_tests(context, tests, cwd="/")
 
 
+@internal_commands.register_subcommand(
+    "dpkg-build-driver-run-task",
+    help_description="[Internal command] Perform a given Dpkg::BuildDriver task (Not stable API)",
+    requested_plugins_only=True,
+    default_log_level=_build_subcommand_log_level,
+    argparser=[
+        add_arg(
+            "task_name",
+            metavar="task-name",
+            choices=[
+                "clean",
+                "build",
+                "build-arch",
+                "build-indep",
+                "binary",
+                "binary-arch",
+                "binary-indep",
+            ],
+            help="The task to run",
+        ),
+        add_arg(
+            "output",
+            nargs="?",
+            default="..",
+            metavar="output",
+            help="Where to place the resulting packages. Should be a directory",
+        ),
+    ],
+)
+def _dpkg_build_driver_integration(context: CommandContext) -> None:
+    parsed_args = context.parsed_args
+    log_level = context.set_log_level_for_build_subcommand()
+    task_name = parsed_args.task_name
+
+    if task_name.endswith("-indep"):
+        context.package_set = "indep"
+    elif task_name.endswith("arch"):
+        context.package_set = "arch"
+
+    manifest = context.parse_manifest()
+
+    plugins = context.load_plugins().plugin_data
+    for plugin in plugins.values():
+        if not plugin.is_bundled:
+            _info(f"Loaded plugin {plugin.plugin_name}")
+    if task_name == "clean":
+        perform_clean(context, manifest)
+    elif task_name in ("build", "build-indep", "build-arch"):
+        perform_builds(context, manifest)
+    elif task_name in ("binary", "binary-indep", "binary-arch"):
+        perform_builds(context, manifest)
+        assemble(
+            context,
+            manifest,
+            INTEGRATION_MODE_FULL,
+            debug_materialization=log_level is not None,
+        )
+    else:
+        _error(f"Unsupported Dpkg::BuildDriver task: {task_name}.")
+
+
 @internal_commands.register_subcommand(
     "dh-integration-generate-debs",
     help_description="[Internal command] Generate .deb/.udebs packages from debian/<pkg> (Not stable API)",
     requested_plugins_only=True,
-    default_log_level=logging.WARN,
+    default_log_level=_build_subcommand_log_level,
     argparser=[
         _add_packages_args,
         add_arg(
@@ -653,17 +733,7 @@ def _run_tests_for_plugin(context: CommandContext) -> None:
 )
 def _dh_integration_generate_debs(context: CommandContext) -> None:
     integrated_with_debhelper()
-    parsed_args = context.parsed_args
-    log_level: Optional[int] = None
-    if os.environ.get("DH_VERBOSE", "") != "":
-        log_level = PRINT_COMMAND
-    if parsed_args.debug_mode:
-        log_level = logging.INFO
-    if log_level is not None:
-        _warn(
-            f"LOG LEVEL: {log_level} -- {logging.WARNING} -- {PRINT_COMMAND} -- {logging.INFO}"
-        )
-        change_log_level(log_level)
+    log_level = context.set_log_level_for_build_subcommand()
     integration_mode = context.resolve_integration_mode()
     is_dh_rrr_only_mode = integration_mode == INTEGRATION_MODE_DH_DEBPUTY_RRR
     if is_dh_rrr_only_mode:
@@ -681,13 +751,28 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
             _info(f"Loaded plugin {plugin.plugin_name}")
     manifest = context.parse_manifest()
 
-    package_data_table = manifest.perform_installations(
-        enable_manifest_installation_feature=not is_dh_rrr_only_mode
+    assemble(
+        context,
+        manifest,
+        integration_mode,
+        debug_materialization=log_level is not None,
     )
+
+
+def assemble(
+    context: CommandContext,
+    manifest: HighLevelManifest,
+    integration_mode: DebputyIntegrationMode,
+    *,
+    debug_materialization: bool = False,
+) -> None:
     source_fs = FSROOverlay.create_root_dir("..", ".")
     source_version = manifest.source_version()
     is_native = "-" not in source_version
-
+    is_dh_rrr_only_mode = integration_mode == INTEGRATION_MODE_DH_DEBPUTY_RRR
+    package_data_table = manifest.perform_installations(
+        enable_manifest_installation_feature=not is_dh_rrr_only_mode
+    )
     if not is_dh_rrr_only_mode:
         for dctrl_bin in manifest.active_packages:
             package = dctrl_bin.name
@@ -705,7 +790,7 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
                 fs_root,
                 dctrl_data.substvars,
             )
-            if "nostrip" not in manifest.build_env.deb_build_options:
+            if "nostrip" not in manifest.deb_options_and_profiles.deb_build_options:
                 dbgsym_ids = relocate_dwarves_into_dbgsym_packages(
                     dctrl_bin,
                     fs_root,
@@ -717,7 +802,7 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
                 dctrl_bin,
                 fs_root,
                 is_native,
-                manifest.build_env,
+                manifest.deb_options_and_profiles,
             )
             if not is_native:
                 install_upstream_changelog(
@@ -742,7 +827,7 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
         manifest,
         package_data_table,
         is_dh_rrr_only_mode,
-        debug_materialization=log_level is not None,
+        debug_materialization=debug_materialization,
     )
 
 
@@ -904,10 +989,10 @@ def _json_output(data: Any) -> None:
     ],
 )
 def _migrate_from_dh(context: CommandContext) -> None:
+    context.must_be_called_in_source_root()
     parsed_args = context.parsed_args
-
     resolved_migration_target = _check_migration_target(
-        context.debian_dir,
+        context,
         parsed_args.migration_target,
     )
     context.debputy_integration_mode = resolved_migration_target
