@@ -15,6 +15,7 @@ from typing import (
     Callable,
     Dict,
     TYPE_CHECKING,
+    Literal,
 )
 
 from debian.debian_support import DpkgArchTable
@@ -45,7 +46,14 @@ from debputy.substitution import (
     SubstitutionImpl,
     NULL_SUBSTITUTION,
 )
-from debputy.util import _error, PKGNAME_REGEX, resolve_source_date_epoch, setup_logging
+from debputy.util import (
+    _error,
+    PKGNAME_REGEX,
+    resolve_source_date_epoch,
+    setup_logging,
+    PRINT_COMMAND,
+    change_log_level,
+)
 
 if TYPE_CHECKING:
     from argparse import _SubParsersAction
@@ -110,6 +118,19 @@ class CommandContext:
                 Mapping[str, "BinaryPackage"],
             ]
         ] = None
+        self._package_set: Literal["both", "arch", "indep"] = "both"
+
+    @property
+    def package_set(self) -> Literal["both", "arch", "indep"]:
+        return self._package_set
+
+    @package_set.setter
+    def package_set(self, new_value: Literal["both", "arch", "indep"]) -> None:
+        if self._dctrl_parser is not None:
+            raise TypeError(
+                "package_set cannot be redefined once the debian/control parser has been initialized"
+            )
+        self._package_set = new_value
 
     @property
     def debian_dir(self) -> VirtualPath:
@@ -135,9 +156,10 @@ class CommandContext:
             parser = DctrlParser(
                 packages,  # -p/--package
                 set(),  # -N/--no-package
-                False,  # -i
-                False,  # -a
-                build_env=DebBuildOptionsAndProfiles.instance(),
+                # binary-indep and binary-indep (dpkg BuildDriver integration only)
+                self._package_set == "indep",
+                self._package_set == "arch",
+                deb_options_and_profiles=DebBuildOptionsAndProfiles.instance(),
                 dpkg_architecture_variables=dpkg_architecture_table(),
                 dpkg_arch_query_table=DpkgArchTable.load_arch_table(),
             )
@@ -152,6 +174,9 @@ class CommandContext:
         _, binary_package_table = self._parse_dctrl()
         return binary_package_table
 
+    def dpkg_architecture_variables(self) -> DpkgArchitectureBuildProcessValuesTable:
+        return self.dctrl_parser.dpkg_architecture_variables
+
     def requested_plugins(self) -> Sequence[str]:
         if self._requested_plugins is None:
             self._requested_plugins = self._resolve_requested_plugins()
@@ -162,7 +187,7 @@ class CommandContext:
 
     @property
     def deb_build_options_and_profiles(self) -> "DebBuildOptionsAndProfiles":
-        return self.dctrl_parser.build_env
+        return self.dctrl_parser.deb_options_and_profiles
 
     @property
     def deb_build_options(self) -> Mapping[str, Optional[str]]:
@@ -292,20 +317,37 @@ class CommandContext:
         debian_control = self.debian_dir.get("control")
         return debian_control is not None
 
-    def resolve_integration_mode(self) -> DebputyIntegrationMode:
+    def resolve_integration_mode(
+        self,
+        require_integration: bool = True,
+    ) -> DebputyIntegrationMode:
         integration_mode = self.debputy_integration_mode
         if integration_mode is None:
             r = read_dh_addon_sequences(self.debian_dir)
             bd_sequences, dr_sequences, _ = r
             all_sequences = bd_sequences | dr_sequences
-            integration_mode = determine_debputy_integration_mode(all_sequences)
-            if integration_mode is None:
+            integration_mode = determine_debputy_integration_mode(
+                self.source_package().fields,
+                all_sequences,
+            )
+            if integration_mode is None and not require_integration:
                 _error(
                     "Cannot resolve the integration mode expected for this package. Is this package using `debputy`?"
                 )
             self.debputy_integration_mode = integration_mode
         return integration_mode
 
+    def set_log_level_for_build_subcommand(self) -> Optional[int]:
+        parsed_args = self.parsed_args
+        log_level: Optional[int] = None
+        if os.environ.get("DH_VERBOSE", "") != "":
+            log_level = PRINT_COMMAND
+        if parsed_args.debug_mode or os.environ.get("DEBPUTY_DEBUG", "") != "":
+            log_level = logging.DEBUG
+        if log_level is not None:
+            change_log_level(log_level)
+        return log_level
+
     def manifest_parser(
         self,
         *,
@@ -320,7 +362,6 @@ class CommandContext:
             manifest_path = self.parsed_args.debputy_manifest
         if manifest_path is None:
             manifest_path = os.path.join(self.debian_dir.fs_path, "debputy.manifest")
-        debian_dir = self.debian_dir
         return YAMLManifestParser(
             manifest_path,
             source_package,
@@ -328,7 +369,7 @@ class CommandContext:
             substitution,
             dctrl_parser.dpkg_architecture_variables,
             dctrl_parser.dpkg_arch_query_table,
-            dctrl_parser.build_env,
+            dctrl_parser.deb_options_and_profiles,
             self.load_plugins(),
             self.resolve_integration_mode(),
             debian_dir=self.debian_dir,
@@ -420,7 +461,7 @@ class GenericSubCommand(SubcommandBase):
         require_substitution: bool = True,
         requested_plugins_only: bool = False,
         log_only_to_stderr: bool = False,
-        default_log_level: int = logging.INFO,
+        default_log_level: Union[int, Callable[[CommandContext], int]] = logging.INFO,
     ) -> None:
         super().__init__(name, aliases=aliases, help_description=help_description)
         self._handler = handler
@@ -452,7 +493,18 @@ class GenericSubCommand(SubcommandBase):
         )
         if self._log_only_to_stderr:
             setup_logging(reconfigure_logging=True, log_only_to_stderr=True)
-        logging.getLogger().setLevel(self._default_log_level)
+
+        default_log_level = self._default_log_level
+        if isinstance(default_log_level, int):
+            level = default_log_level
+        else:
+            assert callable(default_log_level)
+            level = default_log_level(context)
+        change_log_level(level)
+        if level > logging.DEBUG and (
+            context.parsed_args.debug_mode or os.environ.get("DEBPUTY_DEBUG", "") != ""
+        ):
+            change_log_level(logging.DEBUG)
         return self._handler(context)
 
 
@@ -494,7 +546,7 @@ class DispatchingCommandMixin(CommandBase):
         require_substitution: bool = True,
         requested_plugins_only: bool = False,
         log_only_to_stderr: bool = False,
-        default_log_level: int = logging.INFO,
+        default_log_level: Union[int, Callable[[CommandContext], int]] = logging.INFO,
     ) -> Callable[[CommandHandler], GenericSubCommand]:
         if isinstance(name, str):
             cmd_name = name
