@@ -81,7 +81,7 @@ from debputy.util import (
     _error,
     ensure_dir,
     assume_not_none,
-    perl_module_dirs,
+    resolve_perl_config,
     perlxs_api_dependency,
     detect_fakeroot,
     grouper,
@@ -186,11 +186,11 @@ def handle_perl_code(
     fs_root: FSPath,
     substvars: FlushableSubstvars,
 ) -> None:
-    known_perl_inc_dirs = perl_module_dirs(dpkg_architecture_variables, dctrl_bin)
+    perl_config_data = resolve_perl_config(dpkg_architecture_variables, dctrl_bin)
     detected_dep_requirements = 0
 
     # MakeMaker always makes lib and share dirs, but typically only one directory is actually used.
-    for perl_inc_dir in known_perl_inc_dirs:
+    for perl_inc_dir in (perl_config_data.vendorarch, perl_config_data.vendorlib):
         p = fs_root.lookup(perl_inc_dir)
         if p and p.is_dir:
             p.prune_if_empty_dir()
@@ -198,8 +198,8 @@ def handle_perl_code(
     # FIXME: 80% of this belongs in a metadata detector, but that requires us to expose .walk() in the public API,
     #  which will not be today.
     for d, pm_mode in [
-        (known_perl_inc_dirs.vendorlib, PERL_DEP_INDEP_PM_MODULE),
-        (known_perl_inc_dirs.vendorarch, PERL_DEP_ARCH_PM_MODULE),
+        (perl_config_data.vendorlib, PERL_DEP_INDEP_PM_MODULE),
+        (perl_config_data.vendorarch, PERL_DEP_ARCH_PM_MODULE),
     ]:
         inc_dir = fs_root.lookup(d)
         if not inc_dir:
