@@ -17,6 +17,7 @@ from typing import (
     Callable,
     TypeVar,
     Dict,
+    Container,
 )
 
 from debian.deb822 import Deb822
@@ -51,6 +52,8 @@ from debputy.plugin.api import VirtualPath
 from debputy.plugin.api.spec import (
     INTEGRATION_MODE_DH_DEBPUTY_RRR,
     INTEGRATION_MODE_DH_DEBPUTY,
+    DebputyIntegrationMode,
+    INTEGRATION_MODE_FULL,
 )
 from debputy.util import (
     _error,
@@ -61,8 +64,15 @@ from debputy.util import (
     has_glob_magic,
 )
 
+
+class ContainsEverything:
+
+    def __contains__(self, item: str) -> bool:
+        return True
+
+
 # Align with debputy.py
-DH_COMMANDS_REPLACED = {
+DH_COMMANDS_REPLACED: Mapping[DebputyIntegrationMode, Container[str]] = {
     INTEGRATION_MODE_DH_DEBPUTY_RRR: frozenset(
         {
             "dh_fixperms",
@@ -124,6 +134,7 @@ DH_COMMANDS_REPLACED = {
             "dh_builddeb",
         }
     ),
+    INTEGRATION_MODE_FULL: ContainsEverything(),
 }
 
 _GS_DOC = f"{DEBPUTY_DOC_ROOT_DIR}/GETTING-STARTED-WITH-dh-debputy.md"
@@ -375,7 +386,7 @@ def migrate_bash_completion(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_bash-completion files"
     is_single_binary = sum(1 for _ in manifest.all_packages) == 1
@@ -466,7 +477,7 @@ def migrate_dh_installsystemd_files(
     manifest: HighLevelManifest,
     _acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installsystemd files"
     for dctrl_bin in manifest.all_packages:
@@ -499,7 +510,7 @@ def migrate_maintscript(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installdeb files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -608,7 +619,7 @@ def migrate_install_file(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_install config files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -799,7 +810,7 @@ def migrate_installdocs_file(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installdocs config files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -846,7 +857,7 @@ def migrate_installexamples_file(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installexamples config files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -900,7 +911,7 @@ def migrate_installinfo_file(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installinfo config files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -975,7 +986,7 @@ def migrate_installman_file(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installman config files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -1095,7 +1106,7 @@ def migrate_not_installed_file(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_missing's not-installed config file"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -1135,7 +1146,7 @@ def detect_pam_files(
     manifest: HighLevelManifest,
     _acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "detect dh_installpam files (min dh compat)"
     for dctrl_bin in manifest.all_packages:
@@ -1150,7 +1161,7 @@ def migrate_tmpfile(
     manifest: HighLevelManifest,
     _acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_installtmpfiles config files"
     for dctrl_bin in manifest.all_packages:
@@ -1174,7 +1185,7 @@ def migrate_lintian_overrides_files(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_lintian config files"
     for dctrl_bin in manifest.all_packages:
@@ -1198,7 +1209,7 @@ def migrate_links_files(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh_link files"
     mutable_manifest = assume_not_none(manifest.mutable_manifest)
@@ -1272,7 +1283,7 @@ def migrate_misspelled_readme_debian_files(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "misspelled README.Debian files"
     for dctrl_bin in manifest.all_packages:
@@ -1304,7 +1315,7 @@ def migrate_doc_base_files(
     manifest: HighLevelManifest,
     _: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "doc-base files"
     # ignore the dh_make ".EX" file if one should still be present. The dh_installdocs tool ignores it too.
@@ -1355,7 +1366,7 @@ def migrate_dh_hook_targets(
     _: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    migration_target: str,
+    migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "dh hook targets"
     source_root = os.path.dirname(debian_dir.fs_path)
@@ -1407,7 +1418,7 @@ def detect_unsupported_zz_debputy_features(
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "Known unsupported features"
 
@@ -1426,7 +1437,7 @@ def detect_obsolete_substvars(
     _manifest: HighLevelManifest,
     _acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = (
         "Check for obsolete ${foo:var} variables in debian/control"
@@ -1507,7 +1518,7 @@ def detect_dh_addons_zz_debputy_rrr(
     _manifest: HighLevelManifest,
     _acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "Check for dh-sequence-addons"
     r = read_dh_addon_sequences(debian_dir)
@@ -1527,12 +1538,28 @@ def detect_dh_addons_zz_debputy_rrr(
         feature_migration.warn("Missing Build-Depends on dh-sequence-zz-debputy-rrr")
 
 
-def detect_dh_addons(
+def detect_dh_addons_with_full_integration(
+    _debian_dir: VirtualPath,
+    _manifest: HighLevelManifest,
+    _acceptable_migration_issues: AcceptableMigrationIssues,
+    feature_migration: FeatureMigration,
+    _migration_target: DebputyIntegrationMode,
+) -> None:
+    feature_migration.tagline = "Check for dh-sequence-addons and Build-Depends"
+    feature_migration.warn(
+        "TODO: Not implemented: Please remove any dh-sequence Build-Dependency"
+    )
+    feature_migration.warn(
+        "TODO: Not implemented: Please ensure there is a Build-Dependency on `debputy (>= 0.1.45~)"
+    )
+
+
+def detect_dh_addons_with_zz_integration(
     debian_dir: VirtualPath,
     _manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
     feature_migration: FeatureMigration,
-    _migration_target: str,
+    _migration_target: DebputyIntegrationMode,
 ) -> None:
     feature_migration.tagline = "Check for dh-sequence-addons"
     r = read_dh_addon_sequences(debian_dir)
@@ -1544,6 +1571,8 @@ def detect_dh_addons(
         )
         return
 
+    assert _migration_target != INTEGRATION_MODE_FULL
+
     bd_sequences, dr_sequences, _ = r
 
     remaining_sequences = bd_sequences | dr_sequences
