@@ -35,6 +35,8 @@ _KNOWN_NON_TYPO_EXTENSIONS = frozenset(
         "bash",
         "pl",
         "py",
+        # Fairly common image format in older packages
+        "xpm",
     }
 )
 
