@@ -620,7 +620,7 @@ def test_install_rules_no_matches(manifest_parser_pkg_foo) -> None:
         )
     expected_msg = (
         "There were no matches for build/private-arch-tool in /nowhere/debian/tmp, /nowhere"
-        " (definition: installations[0].install <Search for: build/private-arch-tool>)."
+        " (definition: installations[0].install [Line 5 column 6])."
         " Match rule: ./build/private-arch-tool (the exact path / no globbing)"
     )
     assert e_info.value.message == expected_msg
