@@ -15,7 +15,7 @@ from debputy.dh_migration.migrators_impl import (
     migrate_install_file,
     migrate_maintscript,
     migrate_links_files,
-    detect_dh_addons,
+    detect_dh_addons_with_zz_integration,
     migrate_not_installed_file,
     migrate_installman_file,
     migrate_bash_completion,
@@ -1465,7 +1465,7 @@ def test_detect_dh_addons(
     accept_no_migration_issues: AcceptableMigrationIssues,
     accept_any_migration_issues: AcceptableMigrationIssues,
 ) -> None:
-    migrator = detect_dh_addons
+    migrator = detect_dh_addons_with_zz_integration
     empty_fs = build_virtual_file_system([DEBIAN_DIR_ENTRY])
 
     dctrl_no_addons_content = textwrap.dedent(
